
#### Cross Database ForeignKey - MagicField
* 适用场景一：业务拆分为独立服务时, 提供MagicField(弱关系外键)替代ForeignKey, 支持惰性加载，批量加载，ObjCache和RedisCache两层缓存；
* 适用场景二：对象序列化时，提供弱Schema，处理相关Field(包括MagicField);

#### 配置

> settings.py

```python


from django.db import models

from gm_serializer import fields, serializers


class UserManager(object):

    def __init__(self):
        self.url = 'api/user/get'

    def __call__(self, pk_list):
        # mock return user instance from rpc call. 
        return [{'id': pk, 'username': "username %s" % pk} for pk in pk_list]


class Post(models.Model):

    """
    This is demonstrate app model.
    To explain how to use gm-serializer in your project.
    `user` is more like a cross-database foreign-key relation.
    """
    
    user = fields.MagicField(type=int, manager=UserManager, ttl=60*60)
    title = models.CharField(max_length=128)
    content = models.CharField(max_length=255)

    create_at = models.DateTimeField(auto_now_add=True)
    update_at = models.DateTimeField(auto_now=True)


class PostSerializer(serializers.GmModelSerializer):

    user = serializers.MagicField()

    class Meta:
        model = Post
        lazy_fields = ('user',)
        fields = ('user', 'title', 'content', 'create_at')

```

```python

class MagicFieldTestCase(TestCase):

    def setUp(self):
        for i in range(10):
            Post.objects.create(user=i, title="article %s" % i, content="body %s" % i)

    def test_get_object(self):
        post = Post.objects.first()
        self.assertEqual(post.user, {'id': 0, 'username': 'username 0'})

    def test_get_queryset(self):
        """
        pass
        :return:
        """
        posts = Post.objects.all()[:10]
        serializer = PostSerializer(posts, many=True)
        for idx, obj in enumerate(serializer.data):
            self.assertEqual(obj['user'], {'id': idx, 'username': 'username %s' % idx})

    def test_get_queryset_lazy(self):
        posts = Post.objects.all()[:10]
        serializer = PostSerializer(posts, many=True, lazy=True)
        for idx, obj in enumerate(serializer.data):
            self.assertEqual(obj['user'], idx)

```

