from django.core import exceptions
from django.db import models
from django.utils.translation import ugettext_lazy as _

from gm_serializer.fields.descriptor import MagicDescriptor
from gm_serializer.fields.accessor import MagicAccessor

INT = 'INT'
VARCHAR = 'VARCHAR'


class MagicField(models.Field):

    default_error_messages = {
        "invalid": _("'%(value)s' is not a valid %(type)")
    }
    accessor_class = MagicAccessor
    descriptor_class = MagicDescriptor

    def __init__(self, *args, **kwargs):
        """

        :param args:
        :param kwargs:
        """
        self.type = kwargs.pop('type', int)
        self.ttl = kwargs.pop('ttl', None)
        self.manager = kwargs.pop('manager')
        self.accessor = self.accessor_class(
            manager=self.manager,
            ttl=self.ttl
        )
        super(MagicField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "VirtualForeignKey"

    def db_type(self, connection):
        if self.type is int:
            return INT
        elif self.type is str:
            return VARCHAR
        raise RuntimeError("Unsupported data type for VirtualForeignKey: %s" % self.type)

    def get_db_prep_value(self, value, connection, prepared=False):
        """
        lazy foreign key db prep value.
        :param value:
        :param connection:
        :param prepared:
        :return:
        """
        if value is None:
            if self.null:
                return value
            else:
                raise exceptions.ValidationError(
                    self.error_messages['null'],
                    code=['null'],
                    params={'value': value, 'type': str(self.type)}
                )

        if hasattr(value, 'id'):
            return self.type(value.id)

        try:
            return self.type(value)
        except (TypeError, ValueError):
            raise exceptions.ValidationError(
                self.error_messages['invalid'],
                code='invalid',
                params={'value': value, 'type': str(self.type)}
            )

    def contribute_to_class(self, cls, name, **kwargs):
        """
        :param cls:
        :param name:
        :param kwargs:
        :return:
        """
        super(MagicField, self).contribute_to_class(cls, name, **kwargs)
        setattr(cls, self.name, self.descriptor_class(self))

    def deconstruct(self):
        """
        :param
        :return:
        """
        name, path, args, kwargs = super(MagicField, self).deconstruct()
        kwargs.update({
            'type': self.type, 'manager': self.manager,
            'ttl': self.ttl,
        })
        return name, path, args, kwargs

